@@ -9,11 +9,27 @@ use deno_ast::swc::common::SyntaxContext;
 use deno_ast::swc::utils::ident::IdentLike;
 use deno_ast::view::*;
 
+use super::analyze::get_top_level_decls;
+use super::analyze::is_directly_in_condition;
+use super::analyze::is_in_left_hand_assignment;
+use super::analyze::is_in_type;
 use crate::text_changes::TextChange;
 
-const DENO_SHIM_GLOBAL_NAMES: [&str; 12] = [
-  "Blob", "crypto", "Deno", "fetch", "File", "FormData", "Headers", "Request",
-  "Response", "alert", "confirm", "prompt",
+const DENO_SHIM_GLOBAL_NAMES: [&str; 14] = [
+  "Blob",
+  "crypto",
+  "Deno",
+  "fetch",
+  "File",
+  "FormData",
+  "Headers",
+  "Request",
+  "Response",
+  "alert",
+  "confirm",
+  "prompt",
+  "setTimeout",
+  "setInterval",
 ];
 
 pub struct GetDenoGlobalTextChangesParams<'a> {
@@ -35,7 +51,7 @@ pub fn get_deno_global_text_changes(
   params: &GetDenoGlobalTextChangesParams<'_>,
 ) -> Vec<TextChange> {
   let top_level_decls =
-    get_top_level_declarations(params.program, params.top_level_context);
+    get_top_level_decls(params.program, params.top_level_context);
   let ignore_line_indexes = get_ignore_line_indexes(params.program);
   let mut context = Context {
     program: params.program,
@@ -78,7 +94,7 @@ fn visit_children(node: Node, import_name: &str, context: &mut Context) {
     let ident_text = ident.text_fast(context.program);
     if is_top_level_context
       && ident_text == "globalThis"
-      && !should_ignore(ident.into(), context)
+      && !should_ignore_global_this(ident, context)
     {
       context.text_changes.push(TextChange {
         span: ident.span(),
@@ -105,53 +121,70 @@ fn visit_children(node: Node, import_name: &str, context: &mut Context) {
   }
 }
 
+fn should_ignore_global_this(ident: &Ident, context: &Context) -> bool {
+  if should_ignore(ident.into(), context) || is_in_type(ident.into()) {
+    return true;
+  }
+
+  // don't inject the Deno namespace when it's a member expression
+  // not like `globalThis.Deno`
+  if let Some(parent_member_expr) = ident.parent().to::<MemberExpr>() {
+    if parent_member_expr.obj.span().contains(ident.span()) {
+      match parent_member_expr.prop.into() {
+        Node::Ident(prop_ident) => {
+          if prop_ident.sym().as_ref() != "Deno" {
+            return true;
+          }
+        }
+        Node::Str(str) => {
+          if str.value().as_ref() != "Deno" {
+            return true;
+          }
+        }
+        _ => {}
+      }
+    }
+  }
+
+  false
+}
+
 fn should_ignore(node: Node, context: &Context) -> bool {
   context
     .ignore_line_indexes
     .contains(&node.span().start_line_fast(context.program))
-    || in_left_hand_assignment(node)
+    || is_in_left_hand_assignment(node)
     || is_declaration_ident(node)
+    || is_directly_in_condition(node)
 }
 
 fn is_declaration_ident(node: Node) -> bool {
   if let Some(parent) = node.parent() {
-    let span = match parent {
-      Node::BindingIdent(decl) => Some(decl.id.span()),
-      Node::ClassDecl(decl) => Some(decl.ident.span()),
-      Node::ClassExpr(decl) => Some(decl.ident.span()),
-      Node::TsInterfaceDecl(decl) => Some(decl.id.span()),
-      Node::FnDecl(decl) => Some(decl.ident.span()),
-      Node::FnExpr(decl) => Some(decl.ident.span()),
-      Node::TsModuleDecl(decl) => Some(decl.id.span()),
-      Node::TsNamespaceDecl(decl) => Some(decl.id.span()),
-      Node::VarDeclarator(decl) => Some(decl.name.span()),
-      Node::ImportNamedSpecifier(decl) => Some(decl.span()),
-      Node::ExportNamedSpecifier(decl) => Some(decl.span()),
-      Node::ExportNamespaceSpecifier(decl) => Some(decl.span()),
-      Node::ImportDefaultSpecifier(decl) => Some(decl.span()),
-      Node::ExportDefaultSpecifier(decl) => Some(decl.span()),
-      Node::KeyValuePatProp(decl) => Some(decl.key.span()),
-      Node::AssignPatProp(decl) => Some(decl.key.span()),
-      _ => None,
-    };
-    match span {
-      Some(span) => span.contains(node.span()),
-      None => false,
+    match parent {
+      Node::BindingIdent(decl) => decl.id.span().contains(node.span()),
+      Node::ClassDecl(decl) => decl.ident.span().contains(node.span()),
+      Node::ClassExpr(decl) => decl.ident.span().contains(node.span()),
+      Node::TsInterfaceDecl(decl) => decl.id.span().contains(node.span()),
+      Node::FnDecl(decl) => decl.ident.span().contains(node.span()),
+      Node::FnExpr(decl) => decl.ident.span().contains(node.span()),
+      Node::TsModuleDecl(decl) => decl.id.span().contains(node.span()),
+      Node::TsNamespaceDecl(decl) => decl.id.span().contains(node.span()),
+      Node::VarDeclarator(decl) => decl.name.span().contains(node.span()),
+      Node::ImportNamedSpecifier(decl) => decl.span().contains(node.span()),
+      Node::ExportNamedSpecifier(decl) => decl.span().contains(node.span()),
+      Node::ImportDefaultSpecifier(decl) => decl.span().contains(node.span()),
+      Node::ExportDefaultSpecifier(decl) => decl.span().contains(node.span()),
+      Node::ImportStarAsSpecifier(decl) => decl.span().contains(node.span()),
+      Node::ExportNamespaceSpecifier(decl) => decl.span().contains(node.span()),
+      Node::KeyValuePatProp(decl) => decl.key.span().contains(node.span()),
+      Node::AssignPatProp(decl) => decl.key.span().contains(node.span()),
+      _ => false,
     }
   } else {
     false
   }
 }
 
-fn in_left_hand_assignment(node: Node) -> bool {
-  for ancestor in node.ancestors() {
-    if let Node::AssignExpr(expr) = ancestor {
-      return expr.left.span().contains(node.span());
-    }
-  }
-  false
-}
-
 fn get_ignore_line_indexes(program: &Program) -> HashSet<usize> {
   let mut result = HashSet::new();
   for comment in program.comment_container().unwrap().all_comments() {
@@ -169,25 +202,6 @@ fn get_ignore_line_indexes(program: &Program) -> HashSet<usize> {
   result
 }
 
-fn get_top_level_declarations(
-  program: &Program,
-  top_level_context: SyntaxContext,
-) -> HashSet<String> {
-  use deno_ast::swc::common::collections::AHashSet;
-  use deno_ast::swc::utils::collect_decls_with_ctxt;
-  use deno_ast::swc::utils::Id;
-
-  let results: AHashSet<Id> = match program {
-    Program::Module(module) => {
-      collect_decls_with_ctxt(module.inner, top_level_context)
-    }
-    Program::Script(script) => {
-      collect_decls_with_ctxt(script.inner, top_level_context)
-    }
-  };
-  results.iter().map(|v| v.0.to_string()).collect()
-}
-
 fn get_all_ident_names(program: &Program) -> HashSet<String> {
   let mut result = HashSet::new();
   visit_children(&program.into(), &mut result);
