@@ -63,6 +63,8 @@ async fn transform_deno_shim() {
         "const decl09 = alert;\n",
         "const decl10 = confirm;\n",
         "const decl11: typeof prompt = prompt;\n",
+        "setTimeout(() => {}, 100);\n",
+        "setInterval(() => {}, 100);\n",
       ),
       concat!(
         r#"import * as denoShim from "test-shim";"#,
@@ -77,6 +79,8 @@ async fn transform_deno_shim() {
         "const decl09 = denoShim.alert;\n",
         "const decl10 = denoShim.confirm;\n",
         "const decl11: typeof denoShim.prompt = denoShim.prompt;\n",
+        "denoShim.setTimeout(() => {}, 100);\n",
+        "denoShim.setInterval(() => {}, 100);\n",
       ),
     ),
   ])
@@ -105,26 +109,43 @@ async fn transform_deno_shim_with_name_collision() {
 async fn transform_global_this_deno() {
   assert_transforms(vec![
     (
-      "globalThis.Deno.readTextFile(); globalThis.test = 5;",
+      concat!(
+        "globalThis.Deno.readTextFile();",
+        "globalThis.test();",
+        "globalThis.test.test();",
+        "globalThis['test']();",
+        r#"globalThis["test"]();"#,
+        "globalThis.Deno = 5;",
+        "true ? globalThis : globalThis;",
+        "typeof globalThis.Deno;",
+      ),
       concat!(
         r#"import * as denoShim from "test-shim";"#,
-        "\n({ ...denoShim, ...globalThis }).Deno.readTextFile(); globalThis.test = 5;"
+        "\n({ ...denoShim, ...globalThis }).Deno.readTextFile();",
+        "globalThis.test();",
+        "globalThis.test.test();",
+        "globalThis['test']();",
+        r#"globalThis["test"]();"#,
+        "globalThis.Deno = 5;",
+        "true ? ({ ...denoShim, ...globalThis }) : ({ ...denoShim, ...globalThis });",
+        "typeof ({ ...denoShim, ...globalThis }).Deno;"
       )
     ),
   ]).await;
 }
 
 #[tokio::test]
-async fn no_shim_for_declarations() {
+async fn no_shim_situations() {
   assert_identity_transforms(vec![
-    "const { Deno } = test;",
+    "const { Deno } = test; Deno.test;",
+    "const [ Deno ] = test; Deno.test;",
     "const { asdf, ...Deno } = test;",
     "const { Deno: test } = test;",
     "const { test: Deno } = test;",
     "const [Deno] = test;",
     "const [test, ...Deno] = test;",
     "const obj = { Deno: test };",
-    "interface Deno {}",
+    "interface Deno {} function test(d: Deno) {}",
     "interface Test { Deno: string; }",
     "interface Test { Deno(): string; }",
     "class Deno {}",
@@ -143,6 +164,11 @@ async fn no_shim_for_declarations() {
     "export { Deno as test } from 'test';",
     "try {} catch (Deno) {}",
     "function test(Deno) {}",
+    "typeof globalThis;",
+    "globalThis == null;",
+    "globalThis ? true : false;",
+    "type Test = typeof globalThis;",
+    "interface Response {} function test(r: Response) {}",
   ])
   .await;
 }
